VERSION = "0.5.0"

DOMAIN_LIST = [
    "1secmail.com",
    "1secmail.org",
    "1secmail.net",
    "kzccv.com",
    "qiott.com",
    "wuuvo.com",
    "icznn.com",
    "ezztt.com",
]

GEN_RANDOM_MAILBOX = "?action=genRandomMailbox"
GET_DOMAIN_LIST = "?action=getDomainList"
GET_MESSAGES = "?action=getMessages"
GET_SINGLE_MESSAGE = "?action=readMessage"
DELETE_MAILBOX = "?action=deleteMailbox"
DOWNLOAD = "?action=download"
